@@ -178,7 +178,6 @@ IE_Imp_WordPerfect::IE_Imp_WordPerfect(PD_Document * pDocument)
 	m_pCurrentListDefinition(NULL),
 	m_iCurrentListLevel(0),
 	m_bInCell(false),
-	m_iNumCoveredTableCellsInRow(0),
 	m_bHdrFtrOpenCount(0)
 {
 }
@@ -378,6 +377,48 @@ void IE_Imp_WordPerfect::openParagraph(const WPXPropertyList &propList, const WP
 	UT_String_sprintf(tmpBuffer, "; margin-top:%.4fin; margin-bottom:%.4fin; margin-left:%.4fin; margin-right:%.4fin; text-indent:%.4fin; line-height:%.4f",
 		m_topMargin, m_bottomMargin, m_leftMarginOffset, m_rightMarginOffset, m_textIndent, lineSpacing);
 	propBuffer += tmpBuffer;
+	
+	if (tabStops.count() > 0) // Append the tabstop information
+	{
+		propBuffer += "; tabstops:";
+		UT_String_sprintf(tmpBuffer, "");
+                WPXPropertyListVector::Iter i(tabStops);
+                for (i.rewind(); i.next();)
+                {
+			propBuffer += tmpBuffer;
+			if (i()["style:position"])
+			{
+				UT_String_sprintf(tmpBuffer, "%.4fin", i()["style:position"]->getFloat());
+				propBuffer += tmpBuffer;
+			}
+
+			if (i()["style:type"])
+				if (i()["style:type"]->getStr() == "right")
+					propBuffer += "/R";
+				else if (i()["style:type"]->getStr() == "center")
+					propBuffer += "/C";
+				else if (i()["style:type"]->getStr() == "char")
+					propBuffer += "/D";
+				else
+					propBuffer += "/L";
+			else // Left aligned is default
+				propBuffer += "/L";
+
+			if (i()["style:leader-char"])
+				if (i()["style:leader-char"]->getStr() == "-")
+					propBuffer += "2";
+				else if (i()["style:leader-char"]->getStr() == "_")
+					propBuffer += "3";
+				else // default to dot leader if the given leader is dot or is not supported by AbiWord
+					propBuffer += "1";
+			else
+				propBuffer += "0";
+
+			UT_String_sprintf(tmpBuffer, ",");
+                }
+        }
+
+	
 
 	UT_DEBUGMSG(("AbiWordPerfect: Appending paragraph properties: %s\n", propBuffer.c_str()));
 	const XML_Char* propsArray[3];
@@ -851,7 +892,6 @@ void IE_Imp_WordPerfect::openTableRow(const WPXPropertyList &propList)
 	}
 	
 	m_bInCell = false;
-	m_iNumCoveredTableCellsInRow = 0;
 }
 
 void IE_Imp_WordPerfect::openTableCell(const WPXPropertyList &propList)
@@ -875,7 +915,7 @@ void IE_Imp_WordPerfect::openTableCell(const WPXPropertyList &propList)
 	
 	UT_String propBuffer;
 	UT_String_sprintf(propBuffer, "left-attach:%d; right-attach:%d; top-attach:%d; bot-attach:%d",
-					col+m_iNumCoveredTableCellsInRow, col+m_iNumCoveredTableCellsInRow+colSpan, row, row+rowSpan);
+					col, col+colSpan, row, row+rowSpan);
 	
 	UT_String borderStyle;
 	// we only support bg-style:1 for now
@@ -916,7 +956,6 @@ void IE_Imp_WordPerfect::openTableCell(const WPXPropertyList &propList)
 	
 	X_CheckDocumentError(appendStrux(PTX_SectionCell, propsArray));
 	m_bInCell = true;
-	m_iNumCoveredTableCellsInRow += colSpan-1;
 }
 
 void IE_Imp_WordPerfect::closeTable()
